@@ -1,30 +1,41 @@
+from __future__ import annotations
+
+import random
 import re
 import sys
-import random
-from typing import Any, List, Tuple, Union
+from typing import TYPE_CHECKING, Any, List, Tuple, Union
+
+import requests
+from bs4 import BeautifulSoup
 from pyrate_limiter import Duration, Limiter, RequestRate
-import requests
 
-from .utils import (
-    concat_docs,
-    generate_login_cookies,
-    join,
-    soup,
-    get_page_type,
-    get_date,
-    BASE_URL,
-    generate_hash,
-    get,
-    LOGGER,
-    user_agent_list,
-    raise_for_status,
-    LIMITER,
-)
-from .boxes import Thumbnail, Comment, ResultList, _parse_results
-from .pages import Member, Review, Mod, Game, Engine, Group, Team
-from .enums import ThumbnailType, WatchType, Status
-from .errors import ModdbException
 from .base import parse_page
+from .boxes import ResultList, Thumbnail, _parse_results
+from .enums import Status, ThumbnailType
+from .errors import ModdbException
+from .pages import Member
+from .utils import (
+    BASE_URL,
+    LIMITER,
+    LOGGER,
+    concat_docs,
+    generate_hash,
+    generate_login_cookies,
+    get,
+    get_date,
+    get_page_type,
+    get_sitearea,
+    get_siteareaid,
+    join,
+    raise_for_status,
+    soup,
+    user_agent_list,
+)
+
+if TYPE_CHECKING:
+    from .boxes import Comment, Tag
+    from .enums import WatchType
+    from .pages import Engine, Game, Group, Mod, Review, Team
 
 COMMENT_LIMITER = Limiter(RequestRate(1, Duration.MINUTE))
 
@@ -45,7 +56,7 @@ class Message:
         The html text of the message
     """
 
-    def __init__(self, html):
+    def __init__(self, html: BeautifulSoup):
         member = html.find("a", class_="avatar")
 
         self.id = int(html["id"])
@@ -85,7 +96,7 @@ class Thread:
         The messages associated to this thread
     """
 
-    def __init__(self, html):
+    def __init__(self, html: BeautifulSoup):
         thread = html.find("div", id="firstmessage")
         header = thread.find("div", class_="normalcorner")
 
@@ -104,7 +115,8 @@ class Thread:
 
         self.id = int(option["href"][option["href"].index("=") + 1 :])
         self.members = [
-            Thumbnail(url=member["href"], name=member.string, type=ThumbnailType.member) for member in members
+            Thumbnail(url=member["href"], name=member.string, type=ThumbnailType.member)
+            for member in members
         ]
         self.messages = [Message(message) for message in messages]
 
@@ -310,7 +322,9 @@ class Client:
             "User-Agent": random.choice(user_agent_list),
         }
 
-        req = requests.Request(method, url, headers=headers, cookies=cookies, data=kwargs.pop("data", {}))
+        req = requests.Request(
+            method, url, headers=headers, cookies=cookies, data=kwargs.pop("data", {})
+        )
         prepped = self._session.prepare_request(req)
         LOGGER.info("Request: %s", prepped.url)
 
@@ -340,7 +354,8 @@ class Client:
         )
         raw = html.find_all("span", string=strings)
         objects = [
-            e.parent.parent.parent.find("div", class_="table").find_all("div", recursive=False) for e in raw
+            e.parent.parent.parent.find("div", class_="table").find_all("div", recursive=False)
+            for e in raw
         ]
 
         objects_raw = [item for sublist in objects for item in sublist[:-1]]
@@ -408,7 +423,9 @@ class Client:
         html = soup(r.text)
         requests = []
         raw = html.find("span", string="Friend Requests")
-        raw_requests = raw.parent.parent.parent.find("div", class_="table").find_all("div", recursive=False)
+        raw_requests = raw.parent.parent.parent.find("div", class_="table").find_all(
+            "div", recursive=False
+        )
 
         for request in raw_requests[:-1]:
             thumbnail = request.find("a")
@@ -483,7 +500,7 @@ class Client:
             data={
                 "ajax": "t",
                 "action": "watch",
-                "sitearea": page.url.split("/")[-2],
+                "sitearea": get_sitearea(page.url),
                 "siteareaid": page.id,
             },
             allow_redirects=False,
@@ -611,7 +628,7 @@ class Client:
             data={
                 "ajax": "t",
                 "action": "report",
-                "sitearea": page.url.split("/")[-2],
+                "sitearea": get_sitearea(page.url),
                 "siteareaid": page.id,
             },
             allow_redirects=False,
@@ -821,7 +838,9 @@ class Client:
 
         return "Your comment has been saved" in r.json()["text"]
 
-    def add_review(self, page: Any, rating: int, *, text: str = None, has_spoilers: bool = False) -> bool:
+    def add_review(
+        self, page: Any, rating: int, *, text: str = None, has_spoilers: bool = False
+    ) -> bool:
         """Rate and review a page. If you rating is below 3 or above 8 you will be asked
         to also provide a review or else the request will not be made. This is also
         used to edit existing reviews.
@@ -859,7 +878,7 @@ class Client:
             f"{BASE_URL}/reviews/ajax",
             data={
                 "ajax": "t",
-                "sitearea": page.url.split("/")[-2],
+                "sitearea": get_sitearea(page.url),
                 "siteareaid": page.id,
                 "hash": page._review_hash,
                 "earlyaccess": int(page.profile.status == Status.early_access),
@@ -958,7 +977,9 @@ class Client:
         )
         html = soup(r.text)
 
-        threads_raw = html.find_all("div", class_=["tabinbox"])[-1].find_all("div", class_=["rowcontent"])
+        threads_raw = html.find_all("div", class_=["tabinbox"])[-1].find_all(
+            "div", class_=["rowcontent"]
+        )
         threads = []
         for thread in threads_raw:
             member = thread.find("span", class_="subheading").find_all("a")[0]
@@ -1108,3 +1129,46 @@ class Client:
         r = self._request("POST", f"{BASE_URL}/messages/ajax/markallread", data={"ajax": "t"})
 
         return "All messages marked as read" in r.json()["text"]
+
+    def _vote_tag(self, tag: Tag, negative: int):
+        params = {
+            "ajax": "t",
+            "tag": tag.name_id,
+            "sitearea": get_siteareaid(tag.sitearea),
+            "siteareadid": tag.siteareaid,
+            "hash": generate_hash(),
+            "negative": str(negative),
+        }
+
+        resp = self._request("POST", f"{BASE_URL}/tags/ajax/add", data=params)
+        return resp.json()["success"]
+
+    def upvote_tag(self, tag: Tag) -> bool:
+        """Upvote a tag
+
+        Parameters
+        -----------
+        tag : Tag
+            The tag to upvote
+
+        Returns
+        --------
+        bool
+            Whether the upvote was successful
+        """
+        return self._vote_tag(tag, 0)
+
+    def downvote_tag(self, tag: Tag) -> bool:
+        """Downvote a tag
+
+        Parameters
+        -----------
+        tag : Tag
+            The tag to downvote
+
+        Returns
+        --------
+        bool
+            Whether the downvote was successful
+        """
+        return self._vote_tag(tag, 1)
